/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.FreeTextElementFactory;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;

/**
 * Displays the <code>TaxonDescription</code>s attached to a <code>Taxon</code>.
 * <p>
 * For each <code>TaxonDescription</code>:
 * <ul>
 * 	<li>Displays a <code>DescriptionLabelComposite</code> with <code>TaxonDescription#getTextCache()</code></li>
 * 	<li>Displays a <code>DescriptionFeatureGroupComposite</code> for each <code>Feature</code> for which there are descriptive elements</li>
 * 	<li>Displays a <code>DescriptionElementComposite</code> for each <code>TaxonDescription#getElements()</code></li>
 * </ul>
 * </p> 
 * <p>
 * If the <Taxon> has no descriptions, creates a <code>TaxonDescription</code> that is saved when 
 * <code>firePropertyChange()</code> is called with <code>PROP_DIRTY</code>.
 * </p>
 * @author p.ciardelli
 * @created 25.08.2008
 * @version 1.0
 */
public class TaxonDescriptionEditor extends AbstractTaxonEditor 
		implements IDescriptionEditorCompositeRepository {
	private static final Logger logger = Logger
			.getLogger(TaxonDescriptionEditor.class);
	
	/**
	 * If this view is opened and the <code>Taxon</code> has no <code>Description</code>,
	 * a <code>Description</code> element is created and saved to <code>emptyDescription</code>.
	 * It should not be added to the <code>Taxon</code> until it has been changed.
	 * 
	 * @see firePropertyChange(int)
	 */
	private TaxonDescription emptyDescription;
	
	private boolean pageInitialized = false;

	@Override
	public void doSave(IProgressMonitor monitor) {}

	@Override
	public void doSaveAs() {}

	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
				
		super.init(site, input);
		
		Taxon taxon = getTaxon();	
		EditorController.addDescriptionEditor(taxon, this);
	}
	
	@Override
	public boolean isDirty() {
		return false;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	protected void createManagedForm(Composite composite) {
		
		super.createManagedForm(composite);
	
		Taxon taxon = getTaxon();
		
		Set<TaxonDescription> descriptions = taxon.getDescriptions();
		
		// If the taxon has no descriptions, give it an empty one to display
		if (descriptions == null || descriptions.size() == 0) {
			emptyDescription = TaxonDescription.NewInstance();
			TaxonDescription description = emptyDescription;
			description.setTitleCache("");
			descriptions.add(description);
		}
		
		firstGroupedComposite = null;
		
		// Iterate through taxon's descriptions
		for (TaxonDescription description : descriptions) {
			
			// Draw description label
			Composite labelComposite = 
					FreeTextElementFactory.getDefault().createDescriptionLabel(managedForm, taxon, description);
			
			// Note whether this is the first label drawn
			if (firstGroupedComposite == null) {
				firstGroupedComposite = (GroupedComposite) labelComposite;
			}
			
			// Draw each element in description 
			for (DescriptionElementBase element : description.getElements()) {
				FreeTextElementFactory.getDefault().createDescriptionElement(taxon, description, element);
			}
		}
		
		// Redraw composite
		composite.layout();

		pageInitialized = true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#firePropertyChange(int)
	 */
	protected void firePropertyChange(final int propertyId) {
		if (pageInitialized && propertyId == PROP_DIRTY && emptyDescription != null) {
			getTaxon().addDescription(emptyDescription);
			emptyDescription = null;
		}
		super.firePropertyChange(propertyId);
	}
	
	
	public Composite getDescriptionElement(DescriptionElementBase element) {
		for(Composite c : getAllComposites()){
			if(c instanceof DescriptionElementComposite){
				if (element.equals 
						(((DescriptionElementComposite) c).getElement())) {
					return c;
				}
			}
		}
		return null;
	}

	
	public Composite getDescriptionFeatureGroup(TaxonDescription description,
			Feature feature) {
		for(Composite c : getAllComposites()){
			if(c instanceof DescriptionFeatureGroupComposite){
				if (description.equals(((DescriptionFeatureGroupComposite) c).getDescription()) &&
						feature.equals(((DescriptionFeatureGroupComposite) c).getFeature())) {
					return c;
				}
			}
		}
		return null;
	}

	
	public Composite getDescriptionGroup(TaxonDescription description) {
		for(Composite c : getAllComposites()){
			if(c instanceof DescriptionGroupComposite){
				if (description.equals 
						(((DescriptionGroupComposite) c).getDescription())) {
					return c;
				}
			}
		}
		return null;
	}

	
	public Composite getDescriptionLabel(TaxonDescription description) {
		for(Composite c : getAllComposites()){
			if(c instanceof DescriptionLabelComposite){
				if (description.equals 
						(((DescriptionLabelComposite) c).getDescription())) {
					return c;
				}
			}
		}
		return null;
	}
}
