/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.propertysheet;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.PropertySheetEntry;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.controller.EditorController;

/**
 * Redraws editor / freetext area when a value in the property sheet changes.
 * <p>
 * Also includes a lovely little hack to enable custom sorting:
 * <p>
 * GetDisplayName() is called by two methods from the PropertySheetViewer, which
 * <ul>
 *    <li>we are not allowed to set, and</li>
 *    <li>uses a comparator which sorts alphabetically.</li>
 * </ul>
 * So we set display names in the PropertySourceDescriptors as "01:Uninomial",
 *  "02:Specific Epithet"; if we ascertain from the stack trace that a "compare"
 *  method is asking for the display name, we return the uncut version for
 *  sorting; when any other method requests a display name, we chop off the ":"
 *  and everything before it.
 * </p>
 * @author p.ciardelli
 * @created 16.05.2008
 * @version 1.0
 */
public class EditorPropertySheetEntry extends PropertySheetEntry {
	private static final Logger logger = Logger
			.getLogger(EditorPropertySheetEntry.class);
	
	private Taxon taxon;
	
	protected PropertySheetEntry createChildEntry() {
        return new EditorPropertySheetEntry(taxon);
    }
    	    
	public EditorPropertySheetEntry(Taxon taxon) {
		super();
		
		this.taxon = taxon;
	}
	
    /* (non-Javadoc)
     * @see org.eclipse.ui.views.properties.PropertySheetEntry#getDisplayName()
     */
    public String getDisplayName() {
    	
    	String displayName = super.getDisplayName();
    	    	
		Thread currentThread = Thread.currentThread();
		
		// currentThread.getStackTrace()[0] = call to getStackTrace
		// currentThread.getStackTrace()[1] = getDisplayName
		// currentThread.getStackTrace()[2] = who is calling getDisplayName
		StackTraceElement stackRequestingName = currentThread.getStackTrace()[2];

		if (stackRequestingName.toString().lastIndexOf("compare") != -1) {
			return displayName;
		}
		
        return truncateDisplayName(displayName);
    }	
    
    /**
     * Display names take the form "01:xxxx" to allow sorting. This method
     * truncates everything before the colon, and is marked <i>public</i>
     * to allow consistent truncation of property sheet id's.
     * 
     * @param displayName
     * @return
     */
    public static String truncateDisplayName(String displayName) {
		int colon = displayName.lastIndexOf(':');
        if (colon != -1) {
        	displayName = displayName.substring(colon + 1);
        }
    	return displayName;
    }
    
//    protected void valueChanged(PropertySheetEntry child) {
//    	
//    	super.valueChanged(child);
//    	
//    	// If this is the top level of the property sheet, redraw the freetext area
//    	if (getParent() == null) {
//    		EditorController.redraw(taxon);  // causing infinite loops?
//    	}
//    }
    
    protected void refreshFromRoot() {
    	super.refreshFromRoot();
    	
    	// If this is the top level of the property sheet, redraw the freetext area
    	if (getParent() == null) {
    		EditorController.redraw(taxon);  // causing infinite loops?
    	}
    }
}