/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created 19.03.2009
 * @version 1.0
 */
public class TaxonEditorInput implements IEditorInput, IConversationEnabled, IPersistableElement {
	private static final Logger logger = Logger
			.getLogger(TaxonEditorInput.class);

	private Taxon taxon;
	private ConversationHolder conversation;
	
	private TaxonEditorInput(Taxon taxon, ConversationHolder conversation){
		this.taxon = taxon;
		this.conversation = conversation;
	}
	
    /**
     * 
     * @param uuid
     * @param conversation
     * @return
     */
    public static TaxonEditorInput NewInstance(UUID uuid){
    	ConversationHolder conversation = CdmStore.NewTransactionalConversation();
    	
		TaxonBase<?> taxonBase = CdmStore.getTaxonService().getTaxonByUuid(uuid);
		
		if(taxonBase == null){
			MessageDialog.openWarning(EditorUtil.getShell(), "Not yet implemented", "Selected element is not if type TaxonBase.");
			return null;
		}
		
		Taxon taxon;
		
		if (taxonBase instanceof Synonym) {
			// TODO: in case of pro parte synonym or any other where we might have multiple
			// accepted taxa we have to provide a mechanism that can deal with that
			// TODO set focus to the synonym
			Set<Taxon> acceptedTaxa = ((Synonym) taxonBase).getAcceptedTaxa();
			if(acceptedTaxa.size() != 1){
				String message;
				if(acceptedTaxa.size() == 0){
					message = "There is no accepted taxon for the chosen synonym. ";
				}else{
					message = "Multiple taxa found for the chosen synonym.";
				}
				MessageDialog.openWarning(EditorUtil.getShell(), "Not yet implemented", message);
				return null;
			}else{
				taxon = acceptedTaxa.toArray(new Taxon[0])[0];
			}
		} else {
			taxon = (Taxon) taxonBase;
		}	
    	
    	return new TaxonEditorInput(taxon, conversation);
    }
    
    public static TaxonEditorInput NewEmptyInstance(UUID parentTaxonUuid){
    	ConversationHolder conversation = CdmStore.NewTransactionalConversation();
		
		TaxonNameBase name = PreferencesUtil.getInstanceOfPreferredNameClass();
		
		Taxon newTaxon = null;
		if(parentTaxonUuid == null){
			newTaxon = Taxon.NewInstance(name, CdmStore.getDefault().getDefaultSec());
		}else{
			Taxon parentTaxon = (Taxon) CdmStore.getTaxonService().getTaxonByUuid(parentTaxonUuid);
			newTaxon = Taxon.NewInstance(name, parentTaxon.getSec());
			parentTaxon.addTaxonomicChild(newTaxon, null, null);
		}
		// add the new taxon to the editors persistence context
		CdmStore.getTaxonService().save(newTaxon);
		
		return new TaxonEditorInput(newTaxon, conversation);
    }
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	public boolean exists() {
		return taxon != null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	public ImageDescriptor getImageDescriptor() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getName()
	 */
	public String getName() {
		TaxonNameBase name = taxon.getName();
		if (name == null || name.getTitleCache() == null) {
			return "New taxon";
		} else {
			return name.getTitleCache();
		}	
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	public IPersistableElement getPersistable() {
		return this;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getToolTipText()
	 */
	public String getToolTipText() {
		return getName();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class adapter) {

		if (adapter == Taxon.class) {
			return taxon;
		}
		
		return null;
	}
	
    /**
     * Overrides equals to ensure that a taxon can only be edited by
     * one editor at a time.
     * 
     * @return boolean
     */	
	public boolean equals(Object obj) {
		if (obj.getClass().equals(TaxonEditorInput.class) 
				&& taxon.equals(((TaxonEditorInput) obj).taxon))
			return true;
		return false;
	}

	public Taxon getTaxon(){
		return taxon;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostCrudObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmCrudEvent)
	 */
	public void update(CdmDataChangeMap events) {
		// FIXME update the editor input
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPersistableElement#getFactoryId()
	 */
	public String getFactoryId() {
		return TaxonEditorInputFactory.getFactoryId();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IPersistable#saveState(org.eclipse.ui.IMemento)
	 */
	public void saveState(IMemento memento) {
		TaxonEditorInputFactory.saveState(memento, this);
	}
}
