/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.reference;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.reference.INomenclaturalReference;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 13.11.2008
 * @version 1.0
 */
public class ReferenceSearchDialog extends Dialog {
	
	private Table resultsTable;
	private Text searchTermText;
		
	protected Object result;
	private Shell shell;
	private Button okButton;
	private TableViewer resultsTableViewer;
	private ReferenceBase<?> selectedReference;
	private int searchType;
	
	public ReferenceSearchDialog(Shell parent, int searchType) {
		super(parent, SWT.NONE);
		
		this.searchType = searchType;
	}

	public ReferenceSearchDialog(Shell parent) {
		super(parent, SWT.NONE);
	}
	
	/**
	 * Open the dialog
	 * @return the result
	 */
	public Object open() {
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		return result;
	}
	
	/**
	 * Create contents of the dialog
	 */
	protected void createContents() {
		
		// Create shell for popup dialog		
		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setLayout(new GridLayout());
		shell.setSize(500, 375);
		shell.setText("Search for a reference in datasource");
		
		// Create composite for entire shell		
		final Composite composite = new Composite(shell, SWT.NONE);
		composite.setLayout(new GridLayout());
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		// Create composite for search text, search term input, and "Search" button
		final Composite searchComposite = new Composite(composite, SWT.NONE);
		searchComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		searchComposite.setLayout(gridLayout);

		// Create search text
		final Label label = new Label(searchComposite, SWT.NONE);
		label.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
		label.setText("Enter a search term for a full reference, using '*' as a wildcard.");

		// Create input field for search term
		searchTermText = new Text(searchComposite, SWT.BORDER | SWT.SINGLE);
		final GridData gd_searchTermText = new GridData(SWT.FILL, SWT.CENTER, true, false);
		searchTermText.setLayoutData(gd_searchTermText);
		
		// Listen for user hitting <CR> in input field
		searchTermText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				int key = e.keyCode;
				if (key == SWT.CR) {
					populateSearchResults();
				}
			}
		});

		// Create "Search" button
		final Button searchButton = new Button(searchComposite, SWT.NONE);
		searchButton.setLayoutData(new GridData());
		searchButton.setText("Search");
		searchButton.addMouseListener(new MouseAdapter() {
			
			// Populate search results resultsTable after clicking button
			public void mouseUp(MouseEvent e) {
				populateSearchResults();
			}
		});

		// Create composite for results table
		final Composite resultsComposite = new Composite(composite, SWT.NONE);
		resultsComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		resultsComposite.setLayout(new GridLayout());

		// Create results resultsTable
		resultsTableViewer = new TableViewer(resultsComposite, SWT.BORDER);
		resultsTable = resultsTableViewer.getTable();
		resultsTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		// Set content provider of results resultsTable
		resultsTableViewer.setContentProvider(new ArrayContentProvider());

		// Set label provider for results resultsTable which shows ReferenceBase.getTitleCache()
		resultsTableViewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				if (element instanceof ReferenceBase) {
					return ((ReferenceBase<?>) element).getTitleCache();
				}
				return super.getText(element);
			}
		});
	
		// Sort results alphabetically
		resultsTableViewer.setComparator(new ViewerComparator());
		
		// Listen for user selecting reference from results list
		resultsTable.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				
				Object data = e.item.getData();
				
				// Make sure selection is a ReferenceBase
				if (data instanceof ReferenceBase) {

					setSelectedReference((ReferenceBase<?>) data);
					
					// Enable "OK" button
					okButton.setEnabled(true);
				}
			}
		});
		
		// Double-clicking results entry submits selected reference
		resultsTable.addListener(SWT.MouseDoubleClick, new Listener() {
			public void handleEvent(Event event) {
				submitResult();
			}
		});
		
		// Create composite for "OK" and "Cancel" buttons
		final Composite okCancelComposite = new Composite(composite, SWT.NONE);
		okCancelComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
		final GridLayout gridLayout_1 = new GridLayout();
		gridLayout_1.numColumns = 2;
		okCancelComposite.setLayout(gridLayout_1);

		// Create "Cancel" button
		final Button cancelButton = new Button(okCancelComposite, SWT.NONE);
		cancelButton.setText("Cancel");
		
		// Close dialog popup after clicking "Cancel" button
		cancelButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				shell.dispose();
			}
		});

		// Create "OK" button
		okButton = new Button(okCancelComposite, SWT.NONE);
		okButton.setEnabled(false);
		final GridData gd_okButton = new GridData();
		okButton.setLayoutData(gd_okButton);
		okButton.setText("OK");
		
		// Submit result when "OK" button is clicked
		okButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				submitResult();
			}
		});

	}
	
	// FIXME typification very loose
	private void populateSearchResults() {
		// Get search results
		List resultsArray = CdmStore.getReferencesByTitle(searchTermText.getText());

		// Delete non-nomenclatural references as needed
		if (searchType == IReferenceSearch.NOMREF) {
			List nomenclaturalResultsArray = new ArrayList();
			for (Object result : resultsArray) {
				if (result instanceof INomenclaturalReference) {
					nomenclaturalResultsArray.add(result);
				}
			}
			resultsArray = nomenclaturalResultsArray;
		}
		
		// Tell user if there are no results
		if (resultsArray.size() == 0) {
			resultsArray.add("Query returned no results.");
		}
		
		// Send results to results resultsTable
		resultsTableViewer.setInput(resultsArray.toArray());
		
		// Disable OK button
		okButton.setEnabled(false);		
	}
	
	/**
	 * Populate result and close dialog popup
	 */
	private void submitResult() {
		result = getSelectedReference();
		shell.dispose();
	}
	
	private void setSelectedReference(ReferenceBase<?> selectedReference) {
		this.selectedReference = selectedReference;
	}
	
	private ReferenceBase<?> getSelectedReference() {
		return selectedReference;
	}
}
