/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.commands.operations.UndoContext;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.ui.ide.undo.WorkspaceUndoUtil;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.navigator.CommonViewer;

import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.navigation.internal.TaxeditorNavigationPlugin;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.store.model.AbstractUtility;

/**
 * @author n.hoffmann
 * @created 24.03.2009
 * @version 1.0
 */
public class NavigationUtil extends AbstractUtility{
	private static final Logger logger = Logger.getLogger(NavigationUtil.class);
	private static IStatusLineManager statusLineManager;
	private static IUndoContext defaultUndoContext;
	
	/**
	 * FIXME Introduces a dependency on eu.etaxonomy.taxeditor.editor. This is highly undesirable.
	 * 
	 * It is not clear at the moment by what mechanics we are going to open the 
	 * editor. I would prefer some sort of resource model that has the editor linked 
	 * to the resource type of taxonbase. Another reason to switch to CommonNavigatorFamework
	 * 
	 * @param uuid
	 */
	public static void openEditor(IdentifiableEntity selection) {
		try {
			UUID entityUuid = selection.getUuid();			
			EditorUtil.open(entityUuid);
		} catch (PartInitException e) {
			logger.error("Error opening the editor", e);
		}
	}
	
	public static Shell getShell() {
		return getActiveWindow().getShell();
	}

	public static IWorkbenchWindow getActiveWindow() {
		return TaxeditorNavigationPlugin.getDefault().getWorkbench().
				getActiveWorkbenchWindow();
	}
	
	public static void openEmpty(Taxon parentTaxon) {
		try {
			UUID parentTaxonUuid = parentTaxon == null ? null : parentTaxon.getUuid();
			
			EditorUtil.openEmpty(parentTaxonUuid);
		} catch (PartInitException e) {
			logger.error("Error opening the editor", e);
		}
	}
	
	public static IOperationHistory getWorkbenchOperationHistory() {
		return TaxeditorEditorPlugin.getDefault().getWorkbench().
					getOperationSupport().getOperationHistory();
	}
	
	public static IUndoContext getWorkbenchUndoContext() {
		return TaxeditorEditorPlugin.getDefault().getWorkbench().
					getOperationSupport().getUndoContext();
	}
	
	public static void executeOperation(IUndoableOperation operation){
		try {
			IStatus status = getWorkbenchOperationHistory().execute(operation, getMonitor(),
								WorkspaceUndoUtil.getUIInfoAdapter(getShell()));
			String statusString = status.equals(Status.OK_STATUS) ? "completed" : "cancelled";
			setStatusLine(operation.getLabel() + " " + statusString + ".");
		} catch (ExecutionException e) {
			logger.error("Error executing operation: " + operation.getLabel(), e);
		}
	}
	
	private static IProgressMonitor getMonitor() {
		statusLineManager.setCancelEnabled(false);
		return statusLineManager.getProgressMonitor();
	}
	
	public static void setStatusLineManager(IStatusLineManager manager) {
		statusLineManager = manager;
	}

	public static void setStatusLine(String message) {
		statusLineManager.setMessage(message);
	}
	

	/**
	 * Returns the selected taxon for referencing in context menus
	 * 
	 * @param event 
	 * @return
	 */
	public static Taxon getCurrentSelection(ExecutionEvent event){
		
		ISelection menuSelection = HandlerUtil.getActiveMenuSelection(event);
		
		// The selection should always be a tree selection since we are in the 
		// taxonomic tree view. Just in case this will be used in another spot.
		if(menuSelection instanceof TreeSelection){
			Taxon taxon = (Taxon) ((TreeSelection) menuSelection).getFirstElement();
			logger.debug("Selected taxon: " + taxon);
			return taxon;
		}
		return null;
	}

	public static IUndoContext getUndoContext() {
		// FIXME this has to be more specific. Every widget has to have its own undo context
//		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
		
		// Plug-ins that wish their operations to be undoable from workbench views 
		// such as the Navigator or Package Explorer should assign the workbench 
		// undo context to their operations. 
		if (defaultUndoContext == null) {
			defaultUndoContext = new UndoContext();
		}
		return defaultUndoContext;
	}

	public static boolean isDirty(Taxon taxon){
		
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			TaxonEditorInput editorInput;
			try {
				editorInput = (TaxonEditorInput) reference.getEditorInput();
				if(editorInput.getTaxon().equals(taxon) && reference.isDirty()){
					return true;
				}	
			} catch (PartInitException e) {
				e.printStackTrace();
			}
		}
		return false;
	}

	/**
	 * @param parentElement 
	 * @param childTaxon
	 */
	public static void selectInNavigator(Object element, Object parentElement) {
		TaxonNavigator navigator = (TaxonNavigator) TaxeditorEditorPlugin.getDefault().getWorkbench().
				getActiveWorkbenchWindow().getActivePage().findView(TaxonNavigator.ID);
		if (navigator != null) {
			CommonViewer viewer = navigator.getCommonViewer();
			if (viewer != null) {
				if (parentElement != null) {
					viewer.setExpandedState(parentElement, true);
				}
				viewer.setSelection(new StructuredSelection((Taxon) element));
			}
		}
	}
}