/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;
import java.util.UUID;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.impl.TaxonServiceConfiguratorImpl;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.taxeditor.store.datasource.CdmDataSourceRepository;

/**
 * This implementation of ICdmDataRepository depends on hibernate sessions to store the data correctly 
 * for the current session. No state is held in this class.
 * 
 * Only methods that either get or manipulate data are exposed here. So this class acts as a facade 
 * for the methods in cdmlib-service. 
 * 
 * 
 * @author n.hoffmann
 * @created 17.03.2009
 * @version 1.0
 */
public class CdmStore{
	private static final Logger logger = Logger.getLogger(CdmStore.class);
	
	// FIXME change this to ClassPathResources as soon as it is included into the plugin
	private static String DEFAULT_APPLICATION_CONTEXT = null;
	private static DbSchemaValidation DEFAULT_DB_SCHEMA_VALIDATION = DbSchemaValidation.UPDATE;
	
	private static CdmStore instance;
	
	private CdmApplicationController applicationController;
	
	private ConversationHolder globalReadOnlyConversation;
	
	private static DbSchemaValidation dbSchemaValidation;
	
	/**
	 * 
	 * @return
	 */
	public static CdmStore getDefault(){
		return getDefault(DEFAULT_APPLICATION_CONTEXT);
	}
	
	/**
	 * 
	 * @param applicationContextBean
	 * @return
	 */
	public static CdmStore getDefault(String applicationContextBean){
		if(instance == null){
			logger.info("Initializing application context ...");
			ICdmDataSource cdmDatasource = CdmDataSourceRepository.getDefault().
					getCurrentDataSource();
			
			instance = new CdmStore(cdmDatasource, getDbSchemaValidation(), applicationContextBean);
			
			logger.info("Application context initialized.");
		}
		return instance;
	}

	/**
	 * @return
	 */
	private static DbSchemaValidation getDbSchemaValidation() {
		return (dbSchemaValidation == null) ? DEFAULT_DB_SCHEMA_VALIDATION : dbSchemaValidation;
	}

	/**
	 * 
	 */
	private CdmStore(ICdmDataSource dataSource, DbSchemaValidation dbSchemaValidation, String applicationContextBean) {
		// TODO application context bean is not honored by application controller at the moment. 
		// cdmDefaultApplicationController bean gets loaded per default always.
		
		try {
			CdmStore.DEFAULT_APPLICATION_CONTEXT = applicationContextBean;
//			applicationController = CdmApplicationController.NewInstance(applicationContextBean, dataSource, dbSchemaValidation, false);
			
//			logger.warn("OMITTING TERM LOADING FOR DEBUGGING");
//			applicationController = CdmApplicationController.NewInstance(dataSource, dbSchemaValidation, true);
			applicationController = CdmApplicationController.NewInstance(dataSource, dbSchemaValidation);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public Set<TaxonRelationshipType> getConceptRelationshipTypes() {
		Set<TaxonRelationshipType> conceptRelationshipTypes = getTaxonRelationshipTypes().getTerms();
		// remove these two relations as they are considered standard taxon relations
		conceptRelationshipTypes.remove(TaxonRelationshipType.MISAPPLIED_NAME_FOR());
		conceptRelationshipTypes.remove(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN());
	
		return conceptRelationshipTypes;
	}

	public ReferenceBase<?> getDefaultSec() {
		// TODO why is this returning null? and of course, replace w the real deal
		return applicationController.getReferenceService().getReferenceByUuid(
				UUID.fromString("f3593c18-a8d2-4e51-bdad-0befbf8fb2d1"));
	}

	private TermVocabulary<Feature> getFeaturesInternal() {
		return applicationController.getTermService().getVocabulary(VocabularyEnum.Feature);
	}
	
	public static TermVocabulary<Feature> getFeatures() {
		return getDefault().getFeaturesInternal();
	}

	public SortedSet<NameRelationshipType> getNameRelationshipTypes() {
		return applicationController.getTermService().getVocabulary(VocabularyEnum.NameRelationshipType).getTermsOrderedByLabels(Language.DEFAULT());
	}

	public TermVocabulary<NomenclaturalStatusType> getNomStatus() {
		return applicationController.getTermService().getVocabulary(VocabularyEnum.NomenclaturalStatusType);
	}

	private SortedSet<Rank> getRanksInternal() {
		return ((OrderedTermVocabulary)applicationController.getTermService().getVocabulary(VocabularyEnum.Rank)).getOrderedTerms(null);
	}
	
	public static SortedSet<Rank> getRanks() {
		return getDefault().getRanksInternal();
	}

	public List<Taxon> getRootTaxa() {
		boolean onlyWithChildren = false;
		boolean withMisapplications = true;
		
		return applicationController.getTaxonService().getRootTaxa(
				getDefaultSec(), onlyWithChildren, withMisapplications);
	}

	public OrderedTermVocabulary<TaxonRelationshipType> getTaxonRelationshipTypes() {
		return (OrderedTermVocabulary<TaxonRelationshipType>) applicationController.getTermService().getVocabulary(VocabularyEnum.TaxonRelationshipType);
	}

	public Set<Taxon> getTaxonomicChildren(Taxon parentTaxon) {
		new Throwable("Not implemented.");
		return null;
	}
	
	public TermVocabulary<SpecimenTypeDesignationStatus> getSpecimenTypeDesignationStatus() {
		return applicationController.getTermService().getVocabulary(VocabularyEnum.SpecimenTypeDesignationStatus);
	}

	public void removeAllTaxa() {
		new Throwable("Not implemented.");
	}

	public void removeTaxon(Taxon taxon) {
		new Throwable("Not implemented.");
	}

	public UUID saveTaxon(Taxon taxon) {
		return applicationController.getTaxonService().save(taxon);
	}

	public void setApplicationController(
			CdmApplicationController applicationController) {
		this.applicationController = applicationController;
	}
	
	public CdmApplicationController getApplicationController(){
		return applicationController;
	}

	public void setTaxonomicParent(Taxon taxon, Taxon newParentTaxon) {
		new Throwable("Not implemented.");
	}
	
	/**
	 * Create a new conversation and bind resources to it
	 * 
	 * @return
	 */
	public static ConversationHolder NewConversation(){
		
		CdmStore store = getDefault();
		CdmApplicationController controller = store.getApplicationController();
		
		ConversationHolder conversation = controller.NewConversation();

		return conversation;
	}

	/**
	 * Creates a new conversation, binds resources to the conversation and 
	 * start a transaction for this conversation.
	 * 
	 * @return
	 */
	public static ConversationHolder NewTransactionalConversation() {
		ConversationHolder conversation = NewConversation();
		
		conversation.startTransaction();
		return conversation;
	}
	
	
	private ConversationHolder getGlobalReadOnlyConversation() {
		ConversationHolder conversation = globalReadOnlyConversation == null 
				? NewConversation() 
				: globalReadOnlyConversation;
		conversation.bind();
		return conversation;
	}
	
	public static ConversationHolder getGlobalConversation(){
		return getDefault().getGlobalReadOnlyConversation();
	}
	
	public static ITaxonService getTaxonService(){
		return getDefault().getApplicationController().getTaxonService();
	}
	
	public static INameService getNameService(){
		return getDefault().getApplicationController().getNameService();
	}
	
	public static IReferenceService getReferenceService(){
		return getDefault().getApplicationController().getReferenceService();
	}
	
	public static ILocationService getLocationService(){
		return getDefault().getApplicationController().getLocationService();
	}
	
	/**
	 * @param searchText
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static List<TaxonNameBase> searchNameString(String searchText) {
		List<TaxonNameBase> resultSet = new ArrayList<TaxonNameBase>();
		resultSet.addAll(getNameService()
				.getNamesByName(searchText.replace("*", "%")));
		return resultSet;
	}
	
	private List<IdentifiableEntity> findTaxaByName(String queryString, boolean restrictToTaxonObjs) {

		ITaxonServiceConfigurator configurator = new TaxonServiceConfiguratorImpl();
		
		configurator.setSearchString(queryString);
		configurator.setDoTaxa(true);
		configurator.setMatchMode(MatchMode.BEGINNING);
		if (restrictToTaxonObjs) {
			configurator.setDoNamesWithoutTaxa(false);
			configurator.setDoSynonyms(false);			
		} else {
			configurator.setDoNamesWithoutTaxa(true);
			configurator.setDoSynonyms(true);
		}
		configurator.setReferenceBase(null);
		configurator.setPageNumber(0);
		// TODO currently limit results to 1000 for now
		configurator.setPageSize(1000);
		
		List<IdentifiableEntity> result = getTaxonService().findTaxaAndNames(configurator).getRecords();
		
		return result;
	}
	
	public static List<IdentifiableEntity> searchTaxaByName(String queryString){
		return getDefault().findTaxaByName(queryString, false);
	}

	public static List<IdentifiableEntity> searchTaxaByName(String queryString, boolean restrictToTaxonObjs){
		return getDefault().findTaxaByName(queryString, restrictToTaxonObjs);
	}
	
	/**
	 * Searches for references by string. "%" is used as a wildcard.
	 * 
	 * @param text
	 * @return 
	 */
	public static List<ReferenceBase> getReferencesByTitle(String reference) {
		
		reference = reference.replace("*", "%");
		List<ReferenceBase> resultsList = null;
		try {
			resultsList = getReferenceService().getReferencesByTitle(reference);
		} catch (RuntimeException e) {
//			MessageDialog.openError(GlobalController.getShell(), "Search reference error", 
//					"Reference search returned an error. This could be a Hibernate concurrency problem. " +
//					"Please try saving your work, then searching again.");
			e.printStackTrace();
		}
		return resultsList;
	}


	public static List<Language> getLanguages() {
		ITermService termService = getDefault().getApplicationController().getTermService();
		ArrayList<Language> languages = new ArrayList<Language>();
		for (Language language : termService.getLanguageVocabulary()) {
			languages.add(language);
		}
//		languages.add(Language.ENGLISH());
//		languages.add(Language.GERMAN());
		return languages;
	}
	
	/**
	 * FIXME mock
	 * 
	 * @return
	 */
	public static Language getDefaultLanguage(){
		return Language.ENGLISH();
	}
	
	public static boolean changeDataSource(ICdmDataSource dataSource){
		
		// some more things have to happen here

		instance = new CdmStore(dataSource, getDbSchemaValidation(), getApplicationContextBean());
		
		return true;
	}

	/**
	 * @return
	 */
	private static String getApplicationContextBean() {
		return DEFAULT_APPLICATION_CONTEXT;
	}
}
