/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.datasource;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IPath;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.XMLMemento;

import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.CdmPersistentDataSource;
import eu.etaxonomy.cdm.database.DataSourceNotFoundException;
import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

public class CdmDataSourceRepository{
	private static final Logger logger = Logger
			.getLogger(CdmDataSourceRepository.class);

	public static final String TAG_DATASOURCE = "tagDataSource";
	private static final String CURRENT_DATASOURCE = "currentDataSource";
	private static final String DEFAULT_DATASOURCE_STATE_FILENAME = "datasource.xml";
	private static final String DEFAULT_DATASOURCE_NAME = "cdm";
	
	private static ICdmDataSource currentDataSource;
	private static CdmDataSourceRepository repository;
	private XMLMemento memento;
	private String currentDataSourceName;
	
	public CdmDataSourceRepository(){

		
		memento = readMemento();		
		
		
		try {
			currentDataSourceName = memento != null ? memento.getString(CURRENT_DATASOURCE) : DEFAULT_DATASOURCE_NAME;
			currentDataSource = CdmPersistentDataSource.NewInstance(currentDataSourceName);
		} catch (DataSourceNotFoundException e) {
			// fallback creates a new default
			ICdmDataSource h2DataSource = CdmDataSource.NewH2EmbeddedInstance(
					DEFAULT_DATASOURCE_NAME, "sa", "");
			save(h2DataSource);
			setCurrentDataSource(h2DataSource);
		}
		
	}
	
	public static CdmDataSourceRepository getDefault() {
		if (repository == null) {
			repository = new CdmDataSourceRepository();
		}
		return repository;
	}

	public boolean delete(CdmPersistentDataSource dataSource) {
		CdmPersistentDataSource.delete(dataSource);
		return true;
	}

	
	public List<ICdmDataSource> getAll() {
		List<ICdmDataSource> dataSources = new ArrayList<ICdmDataSource>();
		
		for(ICdmDataSource dataSource : CdmPersistentDataSource.getAllDataSources()){
			dataSources.add(dataSource);
		}
		
		return dataSources;
	}
		
	
	public ICdmDataSource getCurrentDataSource() {
		if (currentDataSource == null) {
			throw new IllegalStateException("Current data source not set.");
		}
		return currentDataSource;
	}

	
	public boolean setCurrentDataSource(ICdmDataSource dataSource) {
		if (currentDataSource != null) {
			if (!changeDataSource(dataSource)) {
				return false;
			}
		}
		currentDataSource = dataSource;
		return true;
	}	
		
	private boolean changeDataSource(final ICdmDataSource dataSource) {

		// TODO Close all open editors before showing progress monitor
		if (!StoreUtil.closeAll()) {
			// User has canceled operation   
			return false;
		}
		
		memento.putString(CURRENT_DATASOURCE, dataSource.getName());
		saveMementoToFile(memento);
		PlatformUI.getWorkbench().restart();
	
		return true;
	}

	/**
	 * @param dataSource
	 * @return
	 */
	public ICdmDataSource save(ICdmDataSource dataSource) {
		
		DatabaseTypeEnum databaseType = dataSource.getDatabaseType();
		
		if(databaseType.equals(DatabaseTypeEnum.H2)){
			return CdmPersistentDataSource.saveLocalH2(
					dataSource.getName(), 
					dataSource.getFilePath(), 
					dataSource.getDatabase(), 
					dataSource.getUsername(), 
					dataSource.getPassword(), 
					dataSource.getMode());
		}else{
			return CdmPersistentDataSource.save(
					dataSource.getName(), 
					dataSource.getDatabaseType(),
					dataSource.getServer(),
					dataSource.getDatabase(),
					dataSource.getPort(),
					dataSource.getUsername(),
					dataSource.getPassword()
			);
		}
	}
	
	@Deprecated
	public CdmPersistentDataSource save(String dataSourceName, DatabaseTypeEnum databaseType,
			String server, int port, String database,  String username,
			String password) {
		CdmPersistentDataSource dataSource = CdmPersistentDataSource.save(
				dataSourceName, databaseType, server, database, port, username, password);
		return dataSource;
	}
	
	@Deprecated
	public CdmPersistentDataSource save(String dataSourceName, DatabaseTypeEnum databaseType,
			String server, String database, String username, String password) {
		CdmPersistentDataSource dataSource = CdmPersistentDataSource.save(
				dataSourceName, databaseType, server, database, username, password);
		return dataSource;
	}
	
	/*********************************************************
	 * Memento Handling										 *
	 *********************************************************/
	
	/*
	 * Answer the workbench state file.
	 */
	private File getDataSourceStateFile() {
		IPath path = TaxeditorStorePlugin.getDefault().getStateLocation();
		if (path == null) {
			return null;
		}
		path = path.append(DEFAULT_DATASOURCE_STATE_FILENAME);
		return path.toFile();
	}
	
	private XMLMemento readMemento(){
		File stateFile = getDataSourceStateFile();
		FileInputStream input;
		try {
			input = new FileInputStream(stateFile);
			BufferedReader reader = new BufferedReader(
					new InputStreamReader(input, "utf-8")); //$NON-NLS-1$
			return XMLMemento.createReadRoot(reader);
		} catch (FileNotFoundException e) {
			// first start the file is not there
			return initializeMemento();
			
			// TODO Auto-generated catch block
//			e.printStackTrace();
		} catch (WorkbenchException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}
	
	/**
	 * 
	 */
	private XMLMemento initializeMemento() {
		
		XMLMemento memento = XMLMemento.createWriteRoot(TAG_DATASOURCE);
		memento.putString(CURRENT_DATASOURCE, DEFAULT_DATASOURCE_NAME);
		saveMementoToFile(memento);
		
		return readMemento();
	}

	/*
	 * Save the workbench UI in a persistence file.
	 */
	private boolean saveMementoToFile(XMLMemento memento) {
		// Save it to a file.
		// XXX: nobody currently checks the return value of this method.
		File stateFile = getDataSourceStateFile();
		if (stateFile == null) {
			return false;
		}
		try {
			FileOutputStream stream = new FileOutputStream(stateFile);
			OutputStreamWriter writer = new OutputStreamWriter(stream, "utf-8"); //$NON-NLS-1$
			memento.save(writer);
			writer.close();
		} catch (IOException e) {
			stateFile.delete();
			logger.error("Could not save datasource state");
			return false;
		}

		// Success !
		return true;
	}
	
}