/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;

/**
 * Retrieves the elements of a <code>Taxon</code>'s synonymy list, including misapplied names,
 * using an iterator. 
 * <p> 
 * Note: does not return an entry for the <code>Taxon</code> itself.
 * </p>
 * 
 * @author p.ciardelli
 * @created 09.05.2008
 * @version 1.0
 */
@SuppressWarnings("unchecked")
public class IterableSynonymyList implements Iterable<TaxonBase> {
	
	private Taxon taxon;

	public IterableSynonymyList(Taxon taxon) {
		this.taxon = taxon;
	}

	public Iterator<TaxonBase> iterator() {
		return new TaxonIterator(taxon);
	}
	
	class TaxonIterator implements Iterator<TaxonBase> {

		List<TaxonBase> synonymyList;
		int index = 0;
		
		public TaxonIterator(Taxon taxon) {
			synonymyList = new ArrayList<TaxonBase>();
			HomotypicalGroup homotypicGroup = taxon.getHomotypicGroup();
			
			if (homotypicGroup != null) {
				List<Synonym> homotypicSynonyms = homotypicGroup.getSynonymsInGroup(taxon.getSec());
				for (Synonym synonym : homotypicSynonyms) {

					// Make sure synonym belongs to the taxon
					if (synonym.getAcceptedTaxa().contains(taxon)) {
						synonymyList.add(synonym);
					}
				}
			}
			
			List<HomotypicalGroup> heterotypicGroups = taxon.getHeterotypicSynonymyGroups();
			for (HomotypicalGroup heterotypicGroup : heterotypicGroups) {
				
				// Make sure this is not the taxon's homotypic group
				if (!heterotypicGroup.equals(homotypicGroup)) {
								
					List<Synonym> heterotypicSynonyms = heterotypicGroup.
							getSynonymsInGroup(taxon.getSec());
					for (Synonym synonym : heterotypicSynonyms) {

						// Make sure synonym belongs to the taxon
						if (synonym.getAcceptedTaxa().contains(taxon)) {
							synonymyList.add(synonym);
						}
					}
				}
			}
			
			Set<Taxon> misappliedNames = taxon.getMisappliedNames();
			for (Taxon misappliedName : misappliedNames) {
				synonymyList.add(misappliedName);
			}
			
		}

		public boolean hasNext() {
			if (synonymyList.size() == index) {
				return false;
			}
			return true;
		}

		public TaxonBase<?> next() {
			TaxonBase<?> next = synonymyList.get(index);
			index++;
			return next;
		}

		public void remove() {}
	}
}
