// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.model;

import java.util.HashSet;
import java.util.Set;

import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;

/**
 * @author n.hoffmann
 * @created 22.04.2009
 * @version 1.0
 */
public class SynonymUtil {
	
	/**
	 * Creates a basionym relationship between basionymName and
	 * 	each name in its homotypic group.
	 * 
	 * @param basionymName
	 * @deprecated TODO move this to cmdlib
	 */
	public static void setGroupBasionym(TaxonNameBase<?, ?> basionymName) {

		
		HomotypicalGroup homotypicalGroup = basionymName.getHomotypicalGroup();
		
		if (homotypicalGroup == null) {
			return;
		}
		
		Set<NameRelationship> relations = new HashSet<NameRelationship>();
		Set<NameRelationship> removeRelations = new HashSet<NameRelationship>();
		
		for(TaxonNameBase<?, ?> typifiedName : homotypicalGroup.getTypifiedNames()){
			
			Set<NameRelationship> nameRelations = typifiedName.getRelationsFromThisName();
			
			for(NameRelationship nameRelation : nameRelations){
				relations.add(nameRelation);
			}
		}
		
		for (NameRelationship relation : relations) {
			
			// If this is a basionym relation, and toName is in the homotypical group,
			//	remove the relationship.
			if (relation.getType().equals(NameRelationshipType.BASIONYM()) &&
					relation.getToName().getHomotypicalGroup().equals(homotypicalGroup)) {
				removeRelations.add(relation);
			}
		}
		
		// Removing relations from a set through which we are iterating causes a 
		//	ConcurrentModificationException. Therefore, we delete the targeted
		//	relations in a second step.
		for (NameRelationship relation : removeRelations) {
			basionymName.removeNameRelationship(relation);
		}
		

		for (TaxonNameBase<?, ?> name : homotypicalGroup.getTypifiedNames()) {
			if (!name.equals(basionymName)) {
				
				// First check whether the relationship already exists
				if (!isNameBasionymOf(basionymName, name)) {
					
					// Then create it
					name.addRelationshipFromName(basionymName, 
							NameRelationshipType.BASIONYM(), null);
				}
			}
		}
	}
	
	/**
	 * Creates a basionym relationship between basionymName and
	 * 	each name in its homotypic group.
	 * 
	 * @param basionymName
	 * @deprecated TODO move this to cmdlib
	 */
	public static void removeGroupBasionym(TaxonNameBase<?, ?> basionymName) {

		HomotypicalGroup homotypicalGroup = basionymName.getHomotypicalGroup();
		
		if (homotypicalGroup == null) {
			return;
		}
		
		Set<NameRelationship> relations = new HashSet<NameRelationship>();
		Set<NameRelationship> removeRelations = new HashSet<NameRelationship>();
		
		for(TaxonNameBase<?, ?> typifiedName : homotypicalGroup.getTypifiedNames()){
			
			Set<NameRelationship> nameRelations = typifiedName.getRelationsFromThisName();
			
			for(NameRelationship nameRelation : nameRelations){
				relations.add(nameRelation);
			}
		}
		
		for (NameRelationship relation : relations) {
			
			// If this is a basionym relation, and toName is in the homotypical group,
			//	and fromName is basionymName, remove the relationship.
			if (relation.getType().equals(NameRelationshipType.BASIONYM()) &&
					relation.getFromName().equals(basionymName) &&
					relation.getToName().getHomotypicalGroup().equals(homotypicalGroup)) {
				removeRelations.add(relation);
			}
		}
		
		// Removing relations from a set through which we are iterating causes a 
		//	ConcurrentModificationException. Therefore, we delete the targeted
		//	relations in a second step.
		for (NameRelationship relation : removeRelations) {
			basionymName.removeNameRelationship(relation);
		}
	}
	
	/**
	 * Checks whether synonym's name is the basionym for ALL names 
	 * 	in its group.
	 * 
	 * @param synonym
	 * 
	 * @return
	 * @deprecated TODO move this to cmdlib
	 */
	public static boolean isSynonymGroupBasionym(Synonym synonym) {
		
		TaxonNameBase<?, ?> synonymName = synonym.getName();
		return isNameGroupBasionym(synonymName);
	}
	
	/**
	 * Checks whether name is the basionym for ALL names 
	 * 	in its group.
	 * @param name
	 * 
	 * @return
	 * @deprecated TODO move this to cmdlib
	 */	
	public static boolean isNameGroupBasionym(TaxonNameBase<?, ?> name) {
		if (name == null) {
			return false;
		}
		
		HomotypicalGroup homotypicalGroup = name.getHomotypicalGroup();
		if (homotypicalGroup == null) {
			return false;
		}		
		
		Set<TaxonNameBase> typifiedNames = homotypicalGroup.getTypifiedNames();
		
		// Check whether there are any other names in the group
		if (typifiedNames.size() == 1) {
			return false;
		}
		
				
		for (TaxonNameBase taxonName : typifiedNames) {
			if (!taxonName.equals(name)) {
				if (!isNameBasionymOf(name, taxonName)) {
					return false;
				}
			}
		}
		return true;		
	}
	
	/**
	 * Checks whether a basionym relationship exists between fromName and toName.
	 * 
	 * @param fromName
	 * @param toName
	 * @return
	 * @deprecated TODO move this to cmdlib
	 */
	public static boolean isNameBasionymOf(TaxonNameBase<?, ?> fromName, TaxonNameBase toName) {
		Set<NameRelationship> relations = toName.getRelationsToThisName();
		for (NameRelationship relation : relations) {
			if (relation.getType().equals(NameRelationshipType.BASIONYM()) &&
					relation.getFromName().equals(fromName)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * TODO move this to cdmlib
	 * 
	 * Basically we just want to swap the names. Unfortunately this is all super complicated.
	 * 
	 * @param oldSynonym
	 * @param oldTaxon
	 * @return
	 * 
	 * @deprecated
	 */
	public static Taxon swapSynonymAndAccepted(Synonym oldSynonym, Taxon oldTaxon) {
		
		// Create a new taxon from synonym
		Taxon newTaxon = Taxon.NewInstance(oldSynonym.getName(), oldSynonym.getSec());
		
		// Create a new synonym from the taxon and add it to the newly created taxon
		Synonym newSynonym = Synonym.NewInstance(oldTaxon.getName(), oldTaxon.getSec());			
		
		// Remove synonym from taxon
		oldTaxon.removeSynonym(oldSynonym);
		
		// Swap parent
		Taxon parentTaxon = oldTaxon.getTaxonomicParent();
		newTaxon.setTaxonomicParent(parentTaxon, null, null);
		oldTaxon.setTaxonomicParent(null, null, null);
		
		// Swap children
		Set<Taxon> childTaxa = oldTaxon.getTaxonomicChildren();
		for (Taxon childTaxon : childTaxa) {
			childTaxon.setTaxonomicParent(newTaxon, null, null);
		}
		
		// Swap taxon and synonym
		if (NameUtil.isNameHomotypic(newSynonym.getName(), newTaxon)) {
			newTaxon.addSynonym(newSynonym, SynonymRelationshipType.HOMOTYPIC_SYNONYM_OF());
		} else {
			newTaxon.addSynonym(newSynonym, SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF());			
		}
		
		//	Move the other synonyms from old accepted taxon to new accepted taxon
		for (Synonym synonym : oldTaxon.getSynonyms()) {
			SynonymRelationshipType synRelType = SynonymRelationshipType.HOMOTYPIC_SYNONYM_OF();
			if (NameUtil.isNameHomotypic(synonym.getName(), newTaxon) != true) {
				synRelType = SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF();
			}
			newTaxon.addSynonym(synonym, synRelType);
			oldTaxon.removeSynonym(synonym);
		}
		
		// Move misapplied names
		for (Taxon misappliedNameTaxon : oldTaxon.getMisappliedNames()) {
			newTaxon.addMisappliedName(misappliedNameTaxon, 
					misappliedNameTaxon.getName().getCitation(), 
					""); //TODO: Set the microcitation
			oldTaxon.removeTaxon(misappliedNameTaxon, TaxonRelationshipType.MISAPPLIED_NAME_FOR());
		}
		
		// Move concept relations
		for (TaxonRelationship relation : oldTaxon.getTaxonRelations()) {
			if (relation.getType().equals(TaxonRelationshipType.MISAPPLIED_NAME_FOR()) ||
					relation.getType().equals(TaxonRelationshipType.TAXONOMICALLY_INCLUDED_IN())) {
				continue;
			}
			if (oldTaxon.equals(relation.getFromTaxon())) {
				newTaxon.addTaxonRelation(relation.getToTaxon(), relation.getType(), 
						relation.getCitation(), relation.getCitationMicroReference());
			} else {
				if (relation.getToTaxon() != null) {
					relation.getToTaxon().addTaxonRelation(newTaxon, relation.getType(), 
							relation.getCitation(), relation.getCitationMicroReference());
				}
			}
			oldTaxon.removeTaxonRelation(relation);
		}
		
		// Move descriptions
		Set<TaxonDescription> taxonDescriptions = new HashSet<TaxonDescription>();
		for(TaxonDescription taxonDescription : oldTaxon.getDescriptions()){
			taxonDescriptions.add(taxonDescription);
		}
		for(TaxonDescription taxonDescription : taxonDescriptions){
			newTaxon.addDescription(taxonDescription);
			oldTaxon.removeDescription(taxonDescription);
		}	
		
		return newTaxon;
	}
}
