/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.operations;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.model.TaxonUtil;

/**
 * Delete a taxon from the model.
 * 
 * TODO: Currently all data in session data repository has to be saved before deleting 
 * a taxon and the whole logic is happening here. This way we would have to implement 
 * the same logic in another place if we want to delete outside of an operation context
 * (NOT undoable). Therefore it is desirable to have the logic implemented in another 
 * place and merely call a method here.
 * 
 * @author n.hoffmann
 * @created 16.01.2009
 * @version 1.0
 */
public class DeleteTaxonOperation extends AbstractPostOperation {
	
	@SuppressWarnings("unused")
	private static Logger logger = Logger.getLogger(DeleteTaxonOperation.class);

	/**
	 * The taxonomical parent of the taxon to be deleted.
	 */
	private Taxon parentTaxon;
	
	public DeleteTaxonOperation(String text, IUndoContext undoContext,
			Taxon taxon) {
		super(text, undoContext, taxon, null);

		parentTaxon = taxon.getTaxonomicParent();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		// Get taxon name - watch out for null names
		String taxonName = "";
		if (taxon.getName() != null) {
			taxonName = taxon.getName().getTitleCache();
		}
		
		// Prompt user for confirmation
		if(! MessageDialog.openConfirm((Shell) info.getAdapter(Shell.class), "Confirm Deletion", "Are you sure you want to delete taxon '" + taxonName + "'?")){
			monitor.done();
			return Status.CANCEL_STATUS;
		}
		
		
		// If the taxon has children, cancel operation
		// TODO add option to continue, and delete children
		if (taxon.hasTaxonomicChildren()) {
			MessageDialog.openInformation((Shell) info.getAdapter(Shell.class), "Cannot delete taxon", 
					"'" + taxonName + "' has taxonomic children. " +
					"These must be manually deleted before their parent.");
			monitor.done();
			return Status.CANCEL_STATUS;
		}
		
		// delete the taxon in an isolated conversation
		TaxonUtil.deleteTaxonBaseIsolated(taxon.getUuid());
		
		
		return postExecute(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// FIXME we have to add old citation
		parentTaxon.addTaxonomicChild(taxon, null, null);
		// FIXME readding has to take place in an isolated conversation as well
		// so that all the mediation magic does the rest for us
		return postExecute(null);
	}
}
