/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store.preference;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.SortedSet;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.SourceViewerDecorationSupport;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.store.model.Resources;

/**
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 */
public class PreferencesUtil {
	private static final Logger logger = Logger
			.getLogger(PreferencesUtil.class);

	private static List<Feature> preferredFeatureList;
	private static List<Rank> preferredRankList;

	private static SourceViewerDecorationSupport decorationSupport;

	public static void setNomenclaturalCode(String preferredCode) {
		getPrefStore().setValue(Resources.CODE_PREFERENCE, preferredCode);
	}
	
	public static void setDefaultNomenclaturalCode() {
		setNomenclaturalCode(Resources.DEFAULT_CODE_PREFERENCE);
	}

	public static String getPreferredNomenclaturalCodeAsString() {
		String nameCodePreference = getNameCodePreference();
		
		if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICBN)) {
			return "International Code of Botanical Nomenclature (ICBN)";
		} else if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICZN)) {
			return "International Code of Zoological Nomenclature (ICZN)";
		}
		return null;
	}
	
	/**
	 * @return
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {
	
		String nameCodePreference = getNameCodePreference();
				
		if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICBN)) {
			return NomenclaturalCode.ICBN;
		} else if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICZN)) {
			return NomenclaturalCode.ICZN;
		}
		return null;
	}
	
	private static String getNameCodePreference() {
		try{
			return getPrefStore().getString(Resources.CODE_PREFERENCE);
		}catch(NullPointerException e){
			logger.warn("PreferenceStore was not available. That is OK if you are running a unit test. Setting ICBN as default code.");
			return Resources.CODE_PREFERENCE_ICBN;
		}
	}

	/**
	 * Returns a <code>Set</code> of the <code>Feature</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Feature</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static List<Feature> getPreferredFeatures() {
		
		// Initialize preferredFeatureSet as necessary
		if (preferredFeatureList == null) {
			
			setPreferredFeatures(new ArrayList<Feature>());	
						
			for (Feature feature : CdmStore.getFeatures()) {
				
				// If the feature is set to show, add it to preferredFeatureSet
				if (PreferencesUtil.getFeaturePreference(feature)) {
					addPreferredFeature(feature);
				}
			}
			
			Collections.sort(preferredFeatureList, new Comparator<Feature>(){

				public int compare(Feature feature1, Feature feature2) {
					String feature1Label = feature1.getLabel(CdmStore.getDefaultLanguage());
					String feature2Label = feature2.getLabel(CdmStore.getDefaultLanguage());			
					return feature1Label.compareToIgnoreCase(feature2Label);
				}
				
			});
		}
		
		return preferredFeatureList;
	}

	/**
	 * True if <code>Feature</code> is set to "show" in preferences.
	 * 
	 * @param feature
	 * @return
	 */
	public static boolean getFeaturePreference(Feature feature) {
		
		String preferenceKey = getPreferenceKey(feature);
		
		// If feature does not yet have a pref, set it to true
		if (!PreferencesUtil.getPrefStore().contains(preferenceKey)) {
			PreferencesUtil.getPrefStore().setDefault(preferenceKey, true);
		}
		
		return PreferencesUtil.getPrefStore().getBoolean(preferenceKey);
	}

	/**
	 * Set the show state of a <code>Feature</code> in the 
	 * <code>PreferenceStore</code>.
	 * <p>
	 * Also sets <code>preferredFeatureSet</code> to null to force it be
	 * re-populated the next time {@link getPreferredFeatures} is called.
	 * 
	 * @param feature
	 * @param show
	 */
	public static void setFeaturePreference(Feature feature, boolean show) {
		setPreferredFeatures(null);
		PreferencesUtil.getPrefStore().setValue(PreferencesUtil.getPreferenceKey(feature), show);
	}

	public static IPreferenceStore getPrefStore() {
		return TaxeditorStorePlugin.getDefault().getPreferenceStore();
	}

	/**
	 * Construct a unique key using <code>Feature</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Feature feature) {
		return Resources.FEATURE_PREFERENCE 
				. concat(".")
				. concat(feature.getUuid().toString());
	}

	/**
	 * Returns a <code>Set</code> of the <code>Rank</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Rank</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static List<Rank> getPreferredRanks() {
		
		// Initialize preferredRankSet as necessary
		if (preferredRankList == null) {
			
			setPreferredRanks(new ArrayList<Rank>());	
						
			SortedSet<Rank> ranks = CdmStore.getRanks();
			if (ranks != null) {
			
				for (Rank rank : CdmStore.getRanks()) {
					
					// If the feature is set to show, add it to preferredFeatureSet
					if (PreferencesUtil.getRankPreference(rank)) {
						addPreferredRank(rank);
					}
				}
			} // else { TODO: error message
			
			Collections.sort(preferredRankList, new Comparator<Rank>(){

				public int compare(Rank rank1, Rank rank2) {
					return rank1.isHigher(rank2) ? 1 : 0;
				}	
			});
			
		}
		
		return preferredRankList;
	}

	/**
	 * True if <code>Rank</code> is set to "show" in preferences.
	 * 
	 * @param rank
	 * @return
	 */
	public static boolean getRankPreference(Rank rank) {
		
		String preferenceKey = PreferencesUtil.getPreferenceKey(rank);
		
		// If rank does not yet have a pref, set it to true
		if (!getPrefStore().contains(preferenceKey)) {
			getPrefStore().setDefault(preferenceKey, true);
		}
		
		return getPrefStore().getBoolean(preferenceKey);
	}

	/**
	 * Set the show state of a <code>Rank</code> in the 
	 * <code>PreferenceStore</code>.
	 * <p>
	 * Also sets <code>preferredRankSet</code> to null to force it be
	 * re-populated the next time {@link getPreferredRanks} is called.
	 * 
	 * @param rank
	 * @param show
	 */	
	public static void setRankPreference(Rank rank, boolean show) {
		setPreferredRanks(null);
		getPrefStore().setValue(PreferencesUtil.getPreferenceKey(rank), show);
		
	}

	/**
	 * Construct a unique key using <code>Rank</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Rank rank) {
		return Resources.RANK_PREFERENCE 
				. concat(".")
				. concat(rank.getUuid().toString());
	}

	/**
	 * Returns a new name object that conforms to the nomenclatural
	 * code specified in user preferences. 
	 * 
	 * @return
	 */
	public static NonViralName<?> getInstanceOfPreferredNameClass() {
		NomenclaturalCode code = getPreferredNomenclaturalCode();
		
		// Check whether name code preference needs to be initialized
		if (code == null) {
			setDefaultNomenclaturalCode();
		}
		
		if (code.equals(NomenclaturalCode.ICBN)) {
			return BotanicalName.NewInstance(null);
		} else if (code.equals(NomenclaturalCode.ICZN)) {
			return ZoologicalName.NewInstance(null);			
		}
		return NonViralName.NewInstance(null);
	}

	/**
	 * Appends the code (i.e. ICBN or ICZN) specified in the user preferences
	 * to a string. The result is separated by a period: "<code>key.ICBN</code>".
	 * 
	 * @param key
	 * @return
	 */
	public static String concatCodeMessageSuffix(String key) {
		String code = getNameCodePreference();
		return key + "." + code;
	}
	

	public static void setPreferredFeatures(List<Feature> preferredFeatureList) {
		PreferencesUtil.preferredFeatureList = preferredFeatureList;
	}


	public static void setPreferredRanks(List<Rank> preferredRankList) {
		PreferencesUtil.preferredRankList = preferredRankList;
	}

	public static void addPreferredRank(Rank rank) {
		preferredRankList.add(rank);
	}

	public static void addPreferredFeature(Feature feature) {
		preferredFeatureList.add(feature);		
	}
	
	public static SourceViewerDecorationSupport getDecorationSupport() {
		if (decorationSupport == null) {
			decorationSupport = new SourceViewerDecorationSupport(null, null, null, EditorsUI.getSharedTextColors());
		}
		return decorationSupport;
	}
}
